/*
  Name: Sterownik Astronomiczny
  Copyright: -
  Author: Skalski Lukasz
  Description: ver. 1.01
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <string.h>
#include <math.h>
#include <signal.h>

#define RELAY	"131"								
#define BUTTON	"132"

#define PI   3.14 								//liczba PI
#define req -0.833                    						//stala dla wsch. i zach. - zmierzch cywilny
#define LONG 18.6464                  						//dlugosc geograficzna 
#define LAT 54.3521                  						//szerokosc geograficzna

//ZMIENNE GLOBALNE
time_t czas;
struct tm data;
int year,month,day,hour,minute;
int godzina_wsch, godzina_zach, minuta_wsch, minuta_zach;

//PROTOTYPY FUNKCJI
int export_GPIO(char gpio_pin[]);						//eksport wyprowadzenia do przesrzeni użytkownika
int unexport_GPIO(char gpio_pin[]);						//odeskportowanie wyprowadzenia do jądra
int direction_GPIO(char gpio_pin[], char direction[]);				//ustawienie kierunku "in"/"out"
int set_GPIO(char gpio_pin[], int value);					//ustawienie stanu wyjścia "0"/"1"
int check_GPIO (char gpio_pin[]);

void get_time (void);								//odczyt czasu systemowego
void astro_time (void);								//wyliczenia czasu wschodu i zachodu Słońca
int check (void);								//sprawdzenie warunków załączenia przekaźnika

static void signal_handler (int signo);

int main (int argc, char *argv[]){

	export_GPIO(RELAY);
	export_GPIO(BUTTON);

	direction_GPIO(RELAY,"out");
	direction_GPIO(BUTTON,"in");

	while (1){

		get_time();
		astro_time();

		set_GPIO(RELAY,check());

		if (!check_GPIO(BUTTON)){
			printf ("\nWschod Slonca: %d:%d\n", godzina_wsch,minuta_wsch);
			printf ("Zachod Slonca: %d:%d\n", godzina_zach,minuta_zach);
			printf ("Aktualny czas: %d:%d\n", hour,minute);
			printf ("Stan przekaznika: %d\n", check());
		}

		if (signal (SIGTERM, signal_handler) == SIG_ERR){
			printf ("Blad obslugi sygnalu SIGTERM");
			exit(1);
		}
	
		sleep(1);
	}
return 0;
}


/* ------------------------------------------------------------------------------- */
int export_GPIO(char gpio_pin[]){

	FILE *fd;

	fd = fopen("/sys/class/gpio/export", "w");

	if (!fd){
		fprintf (stderr, "Blad otwarcia pliku\n");
		return -1;
	}else{
		fprintf (fd,"%s", gpio_pin);
	}
	fclose (fd);
	return 0;
}

/* ------------------------------------------------------------------------------- */
int unexport_GPIO(char gpio_pin[]){	

	FILE *fd;
	
	fd = fopen("/sys/class/gpio/unexport", "w");

	if (!fd){
		fprintf (stderr, "Blad otwarcia pliku\n");
		return -1;
	}else{
		fprintf (fd,"%s", gpio_pin);
	}
	fclose (fd);
	return 0;
}

/* ------------------------------------------------------------------------------- */
int direction_GPIO(char gpio_pin[], char direction[]){

	FILE *fd;
	char path [50] ="";

	strcat(path, "/sys/class/gpio/gpio");
	strcat(path, gpio_pin);
	strcat(path, "/direction");

	fd = fopen(path, "w");
	
	if (!fd){
		fprintf (stderr, "Blad otwarcia pliku\n");
		return -1;
	}else{
		fprintf (fd,"%s",direction);
	}
	fclose (fd);
	return 0;
}

/* ------------------------------------------------------------------------------- */
int set_GPIO(char gpio_pin[], int value){

	FILE *fd;
	char path [50] ="";

	strcat(path, "/sys/class/gpio/gpio");
	strcat(path, gpio_pin);
	strcat(path, "/value");

	fd = fopen(path, "w");
	
	if (!fd){
		fprintf (stderr, "Blad otwarcia pliku\n");
		return -1;
	}else{
		fprintf (fd,"%d", value);

	}
	fclose (fd);
	return 0;
}

/* ------------------------------------------------------------------------------- */
int check_GPIO(char gpio_pin[]){

	FILE *fd;
	char path [50] ="";
	char bufor [4];
	int temp;

	strcat(path, "/sys/class/gpio/gpio");
	strcat(path, gpio_pin);
	strcat(path, "/value");

	fd = fopen(path, "r");

	if (!fd){
		fprintf (stderr, "Blad otwarcia pliku\n");
		return -1;
	}else{

		if (!fgets(bufor,3,fd)){
			fprintf (stderr, "Blad odczytu danych\n");
			return -1;
		}else{
			temp = atoi (bufor);
		}
	}
	fclose (fd);
	return temp;
}
/* ------------------------------------------------------------------------------- */
void get_time (void){

    time(&czas);
    localtime_r(&czas,&data);

    year = (data.tm_year) + 1900; 
    month = (data.tm_mon) +1;
    day = data.tm_mday;
    hour = data.tm_hour;
    minute = data.tm_min;
}

/* ------------------------------------------------------------------------------- */
void astro_time (void){

    double temp1,temp2,temp3,temp4;
    int temp_int;                            

    temp1 = (367 * year) - (((((month+9)/12)+year)*7)/4) + ((275*month)/9) + (day-730531.5);
    temp1 = temp1/36525;
    
    temp_int = (4.8949504201433+628.331969753199*temp1) / (6.28318530718); 
    temp2 = (4.8949504201433+628.331969753199*temp1) - (temp_int * 6.28318530718);
                                                                              
    temp_int = (6.2400408+628.3019501*temp1) / (6.28318530718);
    temp3 = (6.2400408+628.3019501*temp1) - (temp3 * 6.28318530718);
    
    temp1 = 0.409093-0.0002269*temp1;
    
    temp4 = 0.033423*sin(temp3)+0.00034907*sin(2*temp3);
    
    temp3 = 0.0430398*sin(2*(temp2+temp4)) - 0.00092502*sin(4*(temp2+temp4)) - temp4;
    
    temp1 = asin(sin(temp1)*sin(temp2+temp4));
    
    temp4 = (sin(0.017453293*req)-sin(0.017453293*LAT)*sin(temp1)) / (cos(0.017453293*LAT)*cos(temp1));
    
    temp1 = (3.14 - (temp3+0.017453293*LONG+1*acos(temp4)))*57.29577951/15;
    
    temp2 = (3.14 - (temp3+0.017453293*LONG+(-1)*acos(temp4)))*57.29577951/15;
    
    minuta_wsch = (int)((temp1 - (int)temp1) * 60) ;   
    minuta_zach = (int)((temp2 - (int)temp2) * 60) ;

    godzina_wsch = (int)temp1 + 2;
    godzina_zach = (int)temp2 + 2;

}
/* ------------------------------------------------------------------------------- */
int check (void){

int temp;

		if (hour == godzina_zach){
			if(minute >= minuta_zach){
				temp = 1;
			}else{
				temp = 0;
			}
		}

		if (hour > godzina_zach){
			temp = 1;
		}


		if (hour < godzina_wsch){
			temp = 1;
		}

		if (hour == godzina_wsch){
			if(minute >= minuta_wsch){
				temp = 0;
			}else{
				temp = 1;
			}
		}

		if ((hour > godzina_wsch) && (hour < godzina_zach)){
			temp = 0;
		}

return temp;
}

/* ------------------------------------------------------------------------------- */
static void signal_handler (int signo){
	unexport_GPIO(RELAY);
	unexport_GPIO(BUTTON);
	exit(1);
}


