/*
  Name: GPS Logger
  Copyright: -
  Author: Skalski Lukasz
  Description: ver. 1.02
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/ioctl.h> 
#include <fcntl.h>
#include <termios.h>
#include <string.h>
#include <signal.h>

//ZMIENNE GLOBALNE
FILE *html;
int fd,licznik;
char buf[80];
char separator[] = ",";

char *schowek;
char *czas;
char *dlugosc_znak;
char *dlugosc;
char *szerokosc_znak;
char *szerokosc;

float wspolrzedna_dlugosc;
float wspolrzedna_szerokosc;

//PROTOTYPY FUNKCJI
void generuj_HTML(void);
void konwertuj_wspolrzedne(void);
static void signal_handler (int signo);


int main(int argc, char *argv[]) { 


	fd = open("/dev/ttyS0", O_RDWR | O_NOCTTY | O_NDELAY);
	html = fopen("Trasa.html", "w"); 

	if (fd < 0){ 
		fprintf(stderr, "Unable to open /dev/ttyS0!\n"); 
		exit(1); 
	}else{

	struct termios options;
	tcgetattr(fd, &options);

	options.c_cflag &= ~CSIZE;
	options.c_cflag |= CS8;
	options.c_cflag &= ~PARENB;
	options.c_cflag &= ~CSTOPB;
	options.c_cflag &= ~CRTSCTS;
	options.c_cflag |= (CLOCAL | CREAD);

	options.c_lflag |= ICANON;
	options.c_lflag &= ~(ECHO | ECHOE);

	options.c_iflag &= ~(IXON | IXOFF | IXANY);
	options.c_iflag &= ~(INLCR | IUCLC);
	options.c_iflag |= (IGNCR);

	options.c_oflag &= ~OPOST;

	cfsetispeed(&options, B4800);
	cfsetospeed(&options, B4800);

	fcntl (fd, F_SETFL, 0);

	tcsetattr(fd, TCSANOW, &options);
	ioctl(fd, TCFLSH,0);
	}

	generuj_HTML();
	while(1){
		
		read(fd,buf,80);

   		schowek = strtok(buf,separator);

		if(strcmp(buf, "$GPGGA") == 0){

				czas = strtok( NULL, separator);
				dlugosc = strtok( NULL, separator);
				dlugosc_znak = strtok( NULL, separator);
				szerokosc = strtok( NULL, separator);
				szerokosc_znak = strtok( NULL, separator);

				if ((strcmp(dlugosc_znak, "N") == 0) || (strcmp(dlugosc_znak, "S") == 0)) {

					konwertuj_wspolrzedne();
					fprintf(html, "var Punkt%d= new GLatLng(%f,%f);\n", licznik,wspolrzedna_dlugosc,wspolrzedna_szerokosc);
					licznik++;
					sleep(5);
				}
		}

		if (signal (SIGTERM, signal_handler) == SIG_ERR){
			printf ("SIGTERM Error");
			exit(1);
		}
		ioctl(fd, TCFLSH,0);
	}
}

/* ------------------------------------------------------------------------------- */
void generuj_HTML (void){

	fprintf(html, "<head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n");
	fprintf(html, "<script src='http://maps.google.com/maps?file=api&v=2' type='text/javascript'></script>\n");
	fprintf(html, "</head><body onload='mapaStart()' onunload='GUnload()'>\n");
	fprintf(html, "<script type='text/javascript'>\n");
	fprintf(html, "<!--\n");
	fprintf(html, "function mapaStart(){\n");
	fprintf(html, "if(GBrowserIsCompatible()){\n");
	fprintf(html, "var mapa = new GMap2(document.getElementById('mapka'),{mapTypes: [G_NORMAL_MAP]});\n");
	fprintf(html, "mapa.addControl(new GLargeMapControl());\n");
	fprintf(html, "mapa.addControl(new GOverviewMapControl());\n");
	fprintf(html, "mapa.addControl(new GScaleControl());\n");
}

/* ------------------------------------------------------------------------------- */
void konwertuj_wspolrzedne(void){

	double dlugosc_f = atof (dlugosc);
	double szerokosc_f = atof (szerokosc);
					
	int stopnie_dlugosc = dlugosc_f / 100;
	int stopnie_szerokosc = szerokosc_f / 100;

	int minuty_dlugosc = (dlugosc_f - stopnie_dlugosc * 100);
	int minuty_szerokosc = (szerokosc_f - stopnie_szerokosc * 100);

	double minuty_d_f = (double)minuty_dlugosc / 60;
	double minuty_s_f = (double)minuty_szerokosc / 60;

	int sekundy_dlugosc = ((dlugosc_f-(stopnie_dlugosc*100+minuty_dlugosc)))*10000;
	int sekundy_szerokosc = ((szerokosc_f-(stopnie_szerokosc*100+minuty_szerokosc)))*10000;		

	double sekundy_d_f = (double)sekundy_dlugosc / 600000;
	double sekundy_s_f = (double)sekundy_szerokosc / 600000;			
				 
	wspolrzedna_dlugosc = stopnie_dlugosc + minuty_d_f + sekundy_d_f;
	wspolrzedna_szerokosc = stopnie_szerokosc + minuty_s_f + sekundy_s_f;
}

/* ------------------------------------------------------------------------------- */
static void signal_handler (int signo){

	int i;

	fprintf(html, "var punkty =[");                 

	for(i = 0 ; i < licznik-1; i++){
		fprintf(html, "Punkt%d,", i);                  
	}

	fprintf(html, "Punkt%d];\n", i);
	fprintf(html, "mapa.setCenter(new GLatLng(%f,%f),12);\n",wspolrzedna_dlugosc,wspolrzedna_szerokosc);
	fprintf(html, "var polilinia = new GPolyline(punkty,'#ff0000',3);\n");
	fprintf(html, "mapa.addOverlay(polilinia);\n");
	fprintf(html, "}\n}\n");
	fprintf(html, "-->\n");
	fprintf(html, "</script>\n");
	fprintf(html, "<div id='mapka' style='width: 800px; height: 600px; border: 2px solid black;'></div>\n");
	fprintf(html, "</div></body></html>\n");

	fclose(html);
	close(fd);
	exit(1);
}
/* ------------------------------------------------------------------------------- */

